/*Made by Khoiidol
YouTube: @khoidol1720
Discord: michaelvzz
@instagram: khoi.real_khoi
*/
import { world, ItemTypes, system } from "@minecraft/server";
import { ActionFormData, ModalFormData, MessageFormData } from "@minecraft/server-ui";
import { saveWarpData, loadWarpData, initializeWarpData } from "./database/database.js";

let { playerWarps, publicWarps, warpAccess, warpCodes } = initializeWarpData();

world.afterEvents.itemUse.subscribe((event) => {
    if (event.itemStack.typeId == "minecraft:compass") {
        openWarpUI(event.source);
    }
});

function openWarpUI(player) {
    const ui = new ActionFormData()
        .title("Warp Menu")
        .button("Create Warp", "textures/ui/ThinPlus.png")
        .button("View Warps", "textures/ui/store_home_icon.png")
        .button("Public Warps", "textures/ui/World.png");

    ui.show(player).then(response => {
        if (response.canceled) return;

        if (response.selection === 0) {
            createWarpUI(player);
        } else if (response.selection === 1) {
            viewWarpsUI(player);
        } else if (response.selection === 2) {
            viewPublicWarpsUI(player); 
        }
    });
}

function createWarpUI(player) {
    const defaultDescription = `by ${player.name}`;

    const createUI = new ModalFormData()
        .title("Create Warp")
        .textField("Warp Name", "Enter warp name")
        .textField("Description of the Warp", "Enter description of the warp", defaultDescription)
        .textField("X Position", "Enter X position", player.location.x.toString())
        .textField("Y Position", "Enter Y position", player.location.y.toString())
        .textField("Z Position", "Enter Z position", player.location.z.toString());

    createUI.show(player).then(response => {
        if (response.canceled) return;

        let warpName = response.formValues[0];
        const description = response.formValues[1];
        const location = {
            x: parseFloat(response.formValues[2]),
            y: parseFloat(response.formValues[3]),
            z: parseFloat(response.formValues[4])
        };

        if (!warpName || warpName.trim() === "") {
            warpName = `${player.name}'s warp`;
        }

        if (!playerWarps[player.name]) {
            playerWarps[player.name] = [];
        }

        playerWarps[player.name].push({
            name: warpName,
            location: location,
            description: description,
            public: false 
        });

        saveWarpData({ playerWarps, publicWarps, warpAccess, warpCodes });

        player.sendMessage(`[ Warps ]: §aWarp§r '${warpName}' §r§acreated at specified location!`);
    });
}


function viewWarpsUI(player) {
    const warps = playerWarps[player.name] || [];

    if (warps.length === 0) {
        player.sendMessage("[ Warps ]: §cYou have no warps.");
        return;
    }

    const actionUI = new ActionFormData()
        .title("Warp Management")
        .button("Show All Warps", "textures/items/compass_item.png")
        .button("Search for Warps", "textures/ui/magnifyingGlass.png");

    actionUI.show(player).then(response => {
        if (response.canceled) return;

        if (response.selection === 0) {
            showAllWarps(player, warps);
        } else if (response.selection === 1) {
            searchForWarps(player, warps);
        }
    });
}

function showAllWarps(player, warps) {
    const viewUI = new ActionFormData().title("Your Warps");

    warps.forEach((warp, index) => {
        viewUI.button(`${warp.name}\n§r${warp.description}`);
    });

    viewUI.show(player).then(response => {
        if (response.canceled) return;

        const selectedWarp = warps[response.selection];
        editWarpUI(player, selectedWarp, response.selection);
    });
}

function searchForWarps(player, warps) {
    const searchUI = new ModalFormData()
        .title("Search Warps")
        .textField("Search by Name or Description", "Enter name or description");

    searchUI.show(player).then(response => {
        if (response.canceled) return;

        const query = response.formValues[0].toLowerCase();
        const filteredWarps = warps.filter(warp => warp.name.toLowerCase().includes(query) || warp.description.toLowerCase().includes(query));

        if (filteredWarps.length === 0) {
            player.sendMessage("[ Warps ]: §cNo warps found matching your search.");
            return;
        }

        showAllWarps(player, filteredWarps);
    });
}

function viewPublicWarpsUI(player) {
    const publicWarpList = Object.values(publicWarps).flat();

    if (publicWarpList.length === 0) {
        player.sendMessage("[ Warps ]: §eThere are no public warps.");
        return;
    }

    const actionUI = new ActionFormData()
        .title("Public Warp Management")
        .button("Show All Public Warps", "textures/ui/icon_multiplayer.png")
        .button("Search for Public Warps", "textures/ui/magnifyingGlass.png");

    actionUI.show(player).then(response => {
        if (response.canceled) return;

        if (response.selection === 0) {
            showAllPublicWarps(player, publicWarpList);
        } else if (response.selection === 1) {
            searchForPublicWarps(player, publicWarpList);
        }
    });
}

function showAllPublicWarps(player, publicWarpList) {
    const publicUI = new ActionFormData().title("Public Warps");

    publicWarpList.forEach((warp, index) => {
        publicUI.button(`${warp.name}\n${warp.description}`);
    });

    publicUI.show(player).then(response => {
        if (response.canceled) return;

        const selectedWarp = publicWarpList[response.selection];
        if (selectedWarp.public && warpCodes[selectedWarp.name]) {
            requestWarpCode(player, selectedWarp);
        } else {
            player.teleport(selectedWarp.location);
            player.sendMessage(`[ Warps ]: §aTeleported to public warp§r '${selectedWarp.name}'.`);
        }
    });
}

function searchForPublicWarps(player, publicWarpList) {
    const searchUI = new ModalFormData()
        .title("Search Public Warps")
        .textField("Search by Name or Description", "Enter name or description");

    searchUI.show(player).then(response => {
        if (response.canceled) return;

        const query = response.formValues[0].toLowerCase();
        const filteredWarps = publicWarpList.filter(warp => warp.name.toLowerCase().includes(query) || warp.description.toLowerCase().includes(query));

        if (filteredWarps.length === 0) {
            player.sendMessage("[ Warps ]: §cNo public warps found matching your search.");
            return;
        }

        showAllPublicWarps(player, filteredWarps);
    });
}

function requestWarpCode(player, warp) {
    const codeRequestUI = new ModalFormData()
        .title(`Enter Code for Warp: ${warp.name}`)
        .textField("Access Code", "Enter the access code for this warp");

    codeRequestUI.show(player).then(response => {
        if (response.canceled) return;

        const enteredCode = response.formValues[0];
        if (warpCodes[warp.name] === enteredCode) {
            player.teleport(warp.location);
            player.sendMessage(`[ Warps ]: §aTeleported to custom warp§r '${warp.name}'.`);
        } else {
            player.sendMessage("[ Warps ]: §cInvalid code. You cannot access this warp.");
        }
    });
}

function editWarpUI(player, warp, warpIndex) {
    const actionForm = new ActionFormData()
        .title(`Manage Warp: ${warp.name}`)
        .button("Teleport to Warp", "textures/items/ender_eye.png")
        .button("Edit Warp", "textures/ui/editIcon.png")
        .button("Delete Warp", "textures/ui/trash.png")
        .button("Advanced Management", "textures/ui/sidebar_icons/menu_threebars.png");

    actionForm.show(player).then(response => {
        if (response.canceled) return;

        if (response.selection === 0) {
            player.teleport(warp.location);
            player.sendMessage(`[ Warps ]: §aTeleported to warp§r '${warp.name}'.`);
        } else if (response.selection === 1) {
            showEditWarpForm(player, warp, warpIndex);
        } else if (response.selection === 2) {
            deleteWarp(player, warpIndex);
        } else if (response.selection === 3) {
            advancedManagementUI(player, warp, warpIndex);
        }
    });
}

function advancedManagementUI(player, warp, warpIndex) {
    const actionForm = new ActionFormData()
        .title(`Advanced Management: ${warp.name}`)
        .button("Share Warp", "textures/gui/newgui/mob_effects/blindness_effect.png")
        .button("Private Warp", "textures/ui/blindness_effect.png");

    actionForm.show(player).then(response => {
        if (response.canceled) return;

        if (response.selection === 0) {
            shareWarpUI(player, warp, warpIndex);
        } else if (response.selection === 1) {
            warp.public = false;
            if (publicWarps[player.name]) {
                publicWarps[player.name] = publicWarps[player.name].filter(w => w.name !== warp.name);
            }
            delete warpCodes[warp.name]; 
            player.sendMessage(`[ Warps ]: §bWarp§r '${warp.name}' §r§bis now private.`);
        }
    });
}

function shareWarpUI(player, warp, warpIndex) {
    const shareForm = new ModalFormData()
        .title(`Share Warp: ${warp.name}`)
        .textField("Code (optional)", "Enter code for custom warp (leave empty for public)")
        .toggle("Is your warp custom?");

    shareForm.show(player).then(response => {
        if (response.canceled) return;

        const accessCode = response.formValues[0];
        const isCustomAccess = response.formValues[1]; 

        if (!publicWarps[player.name]) {
            publicWarps[player.name] = [];
        }

        if (warp.public) {
            publicWarps[player.name] = publicWarps[player.name].filter(w => w.name !== warp.name);
        }

        if (isCustomAccess) {
            warp.public = true;
            warp.description = warp.description.replace(/(by .*?)$/, `§rby ${player.name} - Custom`);
            publicWarps[player.name].push(warp);
            warpAccess[warp.name] = []; 
            warpCodes[warp.name] = accessCode;
            player.sendMessage(`[ Warps ]: §aWarp§r '${warp.name}' §r§ais now public but only accessible with the provided access code.`);
        } else {
            warp.public = true;
            warp.description = warp.description.replace(/(by .*?)$/, `by ${player.name} - Public`);
            publicWarps[player.name].push(warp);
            delete warpCodes[warp.name]; 
            player.sendMessage(`[ Warps ]: §aWarp§r '${warp.name}' §r§ais now public and accessible to everyone.`);
        }

        saveWarpData({ playerWarps, publicWarps, warpAccess, warpCodes });

    }).catch(error => {
        player.sendMessage("§cAn error occurred while updating the warp permissions, try again.");
        console.error("Error updating warp permissions:", error);
    });
}

function showEditWarpForm(player, warp, warpIndex) {
    const editForm = new ModalFormData()
        .title(`Edit Warp: ${warp.name}`)
        .textField("Warp Name", "Enter warp name", warp.name)
        .textField("Description of the Warp", "Enter description of the warp", warp.description)
        .textField("X Position", "Enter X position", warp.location.x.toString())
        .textField("Y Position", "Enter Y position", warp.location.y.toString())
        .textField("Z Position", "Enter Z position", warp.location.z.toString());

    editForm.show(player).then(response => {
        if (response.canceled) return;

        const originalName = warp.name;
        const wasPublic = warp.public;

        if (wasPublic && publicWarps[player.name]) {
            publicWarps[player.name] = publicWarps[player.name].filter(w => w.name !== originalName);
        }

        playerWarps[player.name] = playerWarps[player.name].filter(w => w.name !== originalName);

        warp.name = response.formValues[0];
        warp.description = response.formValues[1];
        warp.location = {
            x: parseFloat(response.formValues[2]),
            y: parseFloat(response.formValues[3]),
            z: parseFloat(response.formValues[4])
        };

        playerWarps[player.name].push(warp);

        if (wasPublic) {
            if (!publicWarps[player.name]) {
                publicWarps[player.name] = [];
            }
            publicWarps[player.name].push(warp);
        }

        if (warpCodes[originalName]) {
            const code = warpCodes[originalName];
            delete warpCodes[originalName];
            warpCodes[warp.name] = code;
        }

        saveWarpData({ playerWarps, publicWarps, warpAccess, warpCodes });

        player.sendMessage(`[ Warps ]: §aWarp§r '${warp.name}' §aupdated successfully!`);

        if (wasPublic) {
            player.sendMessage(`[ Warps ]: §aPublic warp§r '${warp.name}' §aupdated successfully!`);
        }
    }).catch(error => {
        player.sendMessage("§cAn error occurred while updating the warp, please try again.");
        console.error("Error updating warp:", error);
    });
}

function deleteWarp(player, warpIndex) {
    const warpName = playerWarps[player.name][warpIndex].name;

    const confirmForm = new ActionFormData()
        .title("Confirm Deletion")
        .body(`Are you sure you want to delete warp '${warpName}'? §rThis action cannot be undone.`)
        .button("Yes, Delete", "textures/ui/confirm.png")
        .button("Cancel", "textures/ui/cancel.png");

    confirmForm.show(player).then(response => {
        if (response.canceled || response.selection === 1) return; 

        playerWarps[player.name].splice(warpIndex, 1);

        if (publicWarps[player.name]) {
            publicWarps[player.name] = publicWarps[player.name].filter(w => w.name !== warpName);
        }

        delete warpCodes[warpName]; 

        saveWarpData({ playerWarps, publicWarps, warpAccess, warpCodes });

        player.sendMessage(`[ Warps ]: §aWarp§r '${warpName}' §r§adeleted.`);
    });
}